@@ -6,10 +6,10 @@ use warnings;
 
 use AnyEvent;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 require XSLoader;
-XSLoader::load('AnyEvent::YACurl', $VERSION);
+XSLoader::load('AnyEvent::YACurl', '0.07');
 
 require constant;
 my %constants= %{_get_known_constants()};
